@@ -1,6 +1,8 @@
 import asyncio
+from asyncio.log import logger
 import logging
-from time import sleep
+import statistics
+from time import sleep, time
 from serial_asyncio import open_serial_connection
 from .utlis import *
 
@@ -11,9 +13,9 @@ class PemsRequest:
     
     inputs:
         transport (asyncio.BaseTransport): Transport used to for the reply message
-        data (bytearray): data to send on the serial port
+        data (bytes): data to send on the serial port
     '''
-    def __init__(self, transport: asyncio.BaseTransport, data: bytearray) -> None:
+    def __init__(self, transport: asyncio.BaseTransport, data: bytes) -> None:
         self.transport = transport
         self.data = data
 
@@ -43,8 +45,10 @@ class Handler():
         self.writer = None
         self.buffer = b''
         self.timeout_task = None
+        self.start_time_message = None
         self.slaves_timeout_counter = {}
         self.blacklisted_slaves_timer = {}
+        self.message_timers = {}
         
         
         while True:
@@ -65,9 +69,10 @@ class Handler():
 
         self.serial_task = self.loop.create_task(self.run_serial())
         self.unblacklister_task = self.loop.create_task(self.unblacklist_slaves())
+        self.print_info_task = self.loop.create_task(self.print_info())
 
 
-    def send_new_request_from_queue(self):
+    def send_new_request_from_queue(self) -> None:
         ''' Sends the next available request from the queue
         '''
         if self.request_in_progress:
@@ -93,7 +98,7 @@ class Handler():
             self.write_message_to_serial(self.last_request)
         
         
-    def write_message_to_serial(self, p_rquest: PemsRequest):
+    def write_message_to_serial(self, p_rquest: PemsRequest) -> None:
         ''' Writes the request to the serial port
         
         inputs:
@@ -120,7 +125,7 @@ class Handler():
                 self.last_request = None
                 
             
-    async def run_serial(self):
+    async def run_serial(self) -> None:
         ''' Used in a coroutine to open the serial port and read the data
         '''
         while not self.writer:
@@ -139,7 +144,7 @@ class Handler():
                 
         while True:
             try:
-                data = await self.reader.read(1)
+                data = await self.reader.read(100000)
             except Exception:
                 logging.exception()
                 quit()
@@ -147,12 +152,17 @@ class Handler():
             if is_message_complete (self.buffer):
                 logging.debug('new message from serial {}'.format(self.buffer))
                 self.request_in_progress = False
-                self.stop_timeout_timer()
-                slave = get_pems_slave(data)
+                time_passed = self.stop_timeout_timer()
+                slave = get_pems_slave(self.buffer)
+                if slave in self.message_timers:
+                    self.message_timers[slave].append(time_passed)
+                else:
+                    self.message_timers[slave] = [time_passed]
+                
                 self.slaves_timeout_counter[slave] = 0
-                if slave in self.blacklisted_slaves_timer:
-                    self.blacklisted_slaves_timer[slave] = 0
+                if self.blacklisted_slaves_timer.get(slave, 0) > 0:
                     logging.debug('removed slave {} from blacklist beacuse of a new message'.format(slave))
+                self.blacklisted_slaves_timer[slave] = 0
                 try:
                     self.last_request.transport.write(self.buffer)
                 except Exception:
@@ -160,9 +170,10 @@ class Handler():
                 finally:
                     self.buffer = b''
                     self.send_new_request_from_queue()
-                    
+            else:
+                await asyncio.sleep(0.006)
         
-    async def unblacklist_slaves(self):
+    async def unblacklist_slaves(self) -> None:
         ''' Used in a coroutine. Decreases the timer of blacklisted slaves until 0
         '''
         while True:
@@ -170,23 +181,40 @@ class Handler():
                 self.blacklisted_slaves_timer[slave] = max(timer-0.1, 0)
             await asyncio.sleep(0.1)
             
+    
+    async def print_info(self) -> None:
+        ''' Prints periodic information
+        '''
+        while True:
+            await asyncio.sleep(60)
+            logger.info("Slaves information: \n{}".format('\n'.join("slave {} average response time: {}ms, min: {}ms, max: {}ms".format(slave,
+                                                                                                                                     int(statistics.mean(timer)*1000),
+                                                                                                                                     int(min(timer)*1000),
+                                                                                                                                     int(max(timer)*1000)) 
+                                                                     for slave, timer in self.message_timers.items())))
             
-    def start_timeout_timer(self):
+            for timer in self.message_timers.values():
+                timer.clear()
+            
+        
+    def start_timeout_timer(self) -> None:
         ''' Starts timeout timer for the serial message
         '''
         if self.timeout_task:
             self.stop_timeout_timer()
+        self.start_time_message = time()
         self.timeout_task = asyncio.ensure_future(self.timeout())
         
         
-    def stop_timeout_timer(self):
+    def stop_timeout_timer(self) -> float:
         ''' Stops timeout timer for the serial message
         '''
         self.timeout_task.cancel()
         self.timeout_task = None
+        return time() - self.start_time_message
         
         
-    async def timeout(self):
+    async def timeout(self) -> None:
         ''' Handles message timeout
         '''
         await asyncio.sleep(self.serial_timeout/1000)
@@ -225,25 +253,28 @@ class TCPServerProtocol(asyncio.Protocol):
     inputs: handler (Handler)
     '''
     
-    def __init__(self, handler: Handler):
+    def __init__(self, handler: Handler) -> None:
         self.handler = handler
         asyncio.Protocol.__init__(self)
         
     
-    def connection_made(self, transport: asyncio.BaseTransport):
+    def connection_made(self, transport: asyncio.BaseTransport) -> None:
         peername = transport.get_extra_info('peername')
         logging.info('Connection from {}'.format(peername))
         self.transport = transport
-        if self.handler.writer is None:
-            self.transport.close()
             
 
-    def data_received(self, data: bytearray):
+    def data_received(self, data: bytes) -> None:
         logging.debug('new request received: {}'.format(data))
+        
         if get_pems_type(data) == pems_types.CMD_READ_ACCESS_ID:
             self.handle_access_id(get_pems_slave(data))
             return
         
+        if self.handler.writer is None:
+            self.transport.write(create_timeout_message(get_pems_slave(data)))
+            return
+        
         if self.handler.last_request and self.handler.last_request.transport == self.transport:
             # client already asking another request, ignore it
             return
@@ -258,12 +289,12 @@ class TCPServerProtocol(asyncio.Protocol):
         self.handler.send_new_request_from_queue()
         
 
-    def connection_lost(self, exc: Exception):
+    def connection_lost(self, exc: Exception) -> None:
         logging.info('connection lost with client: {}{}'.format(self.transport.get_extra_info('peername'),
                                                          ', error: {}'.format(exc) if exc else ''))
         
         
-    def handle_access_id(self, slave: int):
+    def handle_access_id(self, slave: int) -> None:
         ''' Used to reply to onboard special message
         '''
         try:
