@@ -16,8 +16,8 @@ CMD_READ_ACCESS_ID = 38
 CMD_INFO_OBJ = 44
 CMD_FILE_READ = 101
 CMD_FILE_WRITE = 102
-CMD_TIMEOUT = 150
-CMD_IGNORED = 151
+CMD_TIMEOUT = 150 # FRIGEL CUSTOM
+CMD_IGNORED = 151 # FRIGEL CUSTOM
 CMD_ROM_R_MULTI = 205
 CMD_ANIN_R_MULTI = 207
 CMD_ANOUT_W_MULTI = 208
